--
--
--
--
--

local AnimationBehavior = Game.ecs.behavior:extend('AnimationBehavior', {
	currentAnim = 1,
	animList = {'idle', 'walk'},
	frameTime = 0,
	currentFrame = 1,
	isPaused = true,
	animations = {
		idle = {
			frameRate = 1/12,
			frames = {
				Game.assets.images.boys.chump
			}
		},
		walk = {
			frameRate = 1/12,
			frames = {
				Game.assets.images.boys.chump_walk1,
				Game.assets.images.boys.chump_walk2,
				Game.assets.images.boys.chump_walk3,
			}
		}
	}
})

function AnimationBehavior:wake(dt)
	self.super.update(self)
	if(self.parent.Sprite == nil) then
		print('Attached animation behavior to entity with no Sprite component.')
		self.parent:remove(self.name)
	end
end

function AnimationBehavior:update(dt)
	self.super.update(self)
	
	if(not self.isPaused) then
		self.frameTime = self.frameTime + dt
		
		if(self.frameTime > self.animations[self.animList[self.currentAnim]].frameRate) then
			self.frameTime = 0
			self.currentFrame = self.currentFrame + 1
			
			if(self.currentFrame > #self.animations[self.animList[self.currentAnim]].frames) then
				self.currentFrame = 1
			end
		end
		
		self.parent.Sprite.imageData = self.animations[self.animList[self.currentAnim]].frames[self.currentFrame]
	end
end

function AnimationBehavior:setAnimation(id)
	if(id > #self.animList) then
		return false
	end
	
	if(self.currentAnim ~= id) then
		self.currentAnim = id
		self.currentFrame = 1
		self.frameTime = 0
	end
end

return AnimationBehavior