--
--
--
--
--

local BoyBehavior = Game.ecs.behavior:extend('BoyBehavior', {
	clickCooldown = 0,
	lastClick = 0,
	decayRate = 0.5,
	decayTimer = 0,
	hatchButton = nil,
	clicked = false,
	shakeDir = 1,
	evolvePop = 0,
	
	direction = 0,
	
	hungerSaturation = 0.2,
	hungerTimer = 0,
	
	stateAge = 0,
	lastBite = 0,
	
	evolveButtons = {},
	evolving = false,
	
	state = 0,
	pastStates = {}
})

function BoyBehavior:wake(dt)
	self.parent.hitbox.width = self.parent.Sprite.imageData:getWidth()
	self.parent.hitbox.height = self.parent.Sprite.imageData:getHeight()
	
	self.parent.hitbox.offset.x = self.parent.hitbox.offset.x - math.floor((self.parent.Sprite.imageData:getWidth()) * (1 - self.parent.Sprite.origin.x))
	self.parent.hitbox.offset.y = self.parent.hitbox.offset.y - math.floor((self.parent.Sprite.imageData:getHeight()) * (self.parent.Sprite.origin.y))
end

function BoyBehavior:update(dt)	
	if(self.clickCooldown > 0) then
		self.clickCooldown = self.clickCooldown - 1
	end
	
	if(Game.state.clicks < 50 or (self.parent.stats.boyState ~= Game.definitions.BoyState.EGG)) then
		if(self.parent.Sprite.rotation > 0) then
			self.parent.Sprite.rotation = self.parent.Sprite.rotation - math.rad(1)
		end
		
		if(self.parent.Sprite.rotation < 0) then
			self.parent.Sprite.rotation = self.parent.Sprite.rotation + math.rad(1)
		end
		
		self.isShaking = false
	else
		if(not self.isShaking) then
			self.isShaking = true
			self.parent.Sprite.rotation = 0
		end
		
		if(math.abs(self.parent.Sprite.rotation) >= math.rad(5)) then
			self.shakeDir = -self.shakeDir
		end
		
		self.parent.Sprite.rotation = self.parent.Sprite.rotation + (self.shakeDir * math.rad(1))
	end
	
	if(math.abs(self.parent.Sprite.rotation) < math.rad(1)) then
		self.parent.Sprite.rotation = 0
	end
	
	if(self.parent.mode.clicked) then
		self.lastClick = 0
	else
		self.lastClick = self.lastClick + dt
	end
	
	if(self.parent.stats.boyState ~= Game.definitions.BoyState.EGG) then
		self:manageMoods()
		self:manageState(dt)
		self:resolveState(dt)
	end
	
	self:status()
end

function BoyBehavior:draw(dt)
	local lastLayer = love.graphics.getCanvas()
	
	love.graphics.setCanvas(Game.layers.foreground)
	
	if(self.parent.stats.boyState == Game.definitions.BoyState.EGG) then
		if(self.parent.mode.clicked or self.clickCooldown > 0) then
			love.graphics.draw(Game.assets.images.eye_close, 
				math.round(self.parent.x - 30),
				math.round(self.parent.y - 85)
			)
			
			if(self.parent.mode.clicked) then
				self.clickCooldown = 10
			end
		else
			love.graphics.draw(Game.assets.images.eye_open, 
				math.round(self.parent.x - 30),
				math.round(self.parent.y - 90)
			)
		end
		
		if(self.parent.y == Game.state.floor_y) then
			love.graphics.draw(Game.assets.images.leaf, self.parent.x - math.round(self.parent.hitbox.width / 2), self.parent.y - 40)
		end
	end
	
	love.graphics.setCanvas(lastLayer)
end

function BoyBehavior:manageState(dt)
	self.stateAge = self.stateAge + dt
	
	local stats = self.parent.stats.emotions
	
	if(stats.hunger < 30 and self.state ~= Game.definitions.Actions.FEEDING) then
		self:setAction(Game.definitions.Actions.PECKISH)
	end
	
	if(self.evolving) then
		self:setAction(Game.definitions.Actions.EVOLVE)
	end
end

function BoyBehavior:setAction(presentedState)
	if(self.state ~= presentedState) then
		table.insert(self.pastStates, self.state)
		self.state = presentedState
		self.stateAge = 0
	end
end

function BoyBehavior:resolveState(dt)
	if(self.state == Game.definitions.Actions.IDLE) then
		if(self.direction == -1) then
			self.parent.velocity.x = -2
			self.parent:get('AnimationBehavior'):setAnimation(2)
		elseif(self.direction == 0) then
			self.parent.velocity.x = 0
			self.parent:get('AnimationBehavior'):setAnimation(1)
		else
			self.parent.velocity.x = 2
			self.parent:get('AnimationBehavior'):setAnimation(2)
		end
		
		if(self.stateAge > 1) then
			self.stateAge = 0
			self.direction = math.random(-1, 1)
		end
	elseif(self.state == Game.definitions.Actions.PECKISH) then		
		local nearestFood = 1000
		self.eating = nil
		
		if(Game.state.foodCount > 0) then
			for index, food in pairs(Game.state.food) do
				local foodObj = food
				
				local _distance = math.dist(food.x, food.y, self.parent.x, self.parent.y)
				if(math.abs(_distance) < nearestFood or self.eating == nil) then
					nearestFood = math.abs(_distance)
					self.eating = food
				end
			end
		end
		
		if(self.eating == nil) then
			--
			-- No nearby food!
			--
			if(self.timeWithoutFood == nil) then
				self.timeWithoutFood = 0
			else
				self.timeWithoutFood = self.timeWithoutFood + dt
			end
			
			if(self.timeWithoutFood > 5) then
				self.hungerTimer = self.hungerTimer + dt
				
				if(self.hungerTimer > 3) then
					self.hungerTimer = 0
					popOff(Game.strings.Messages.HUNGRY, self.parent.x, self.parent.y)
				end
			end
			
			self:setAction(Game.definitions.BoyState.IDLE)
			self.parent:get('AnimationBehavior'):setAnimation(1)
		else
			Game.particles:moveTo(self.eating.x, self.eating.y)
			Game.particles:emit(50)
			if(self.eating.x < self.parent.x) then
				self.parent.velocity.x = -3
				self.parent:get('AnimationBehavior'):setAnimation(2)
			else
				self.parent.velocity.x = 3
				self.parent:get('AnimationBehavior'):setAnimation(2)
			end
			
			if(self.eating.Hitbox ~= nil and self.parent.Hitbox ~= nil) then
				if(self.parent.Hitbox.shape:collidesWith(self.eating.Hitbox.shape)) then
					self:setAction(Game.definitions.Actions.FEEDING)
					self.parent.velocity.x = 0
					self.timeWithoutFood = 0
				end
			else 
				self.eating = nil
				self:setAction(Game.definitions.Actions.IDLE)
			end
		end
	elseif(self.state == Game.definitions.Actions.FEEDING) then
		if(self.eating == nil) then
			self:setAction(Game.definitions.Actions.IDLE)
			self.parent:get('AnimationBehavior'):setAnimation(1)
		else
			self.lastBite = self.lastBite + dt
			
			if(self.lastBite > 0.5) then
				self.lastBite = 0
				local remains = self.eating:get(self.eating.foodBehavior):onBite(self.parent)

				if(remains <= 0 or self.parent.stats.emotions.hunger > 50) then
					self.eating = nil
					self:setAction(Game.definitions.Actions.IDLE)
				end
			end
			
			if(self.parent.stats.emotions.hunger > 80) then
				self.eating = nil
				self:setAction(Game.definitions.Actions.IDLE)	
				self.parent:get('AnimationBehavior'):setAnimation(1)
			end
		end
	elseif(self.state == Game.definitions.Actions.EVOLVE) then
		self.parent.velocity.y = -15
		self.evolving = false
	end
end

function BoyBehavior:manageMoods()
	self.decayTimer = self.decayTimer + love.timer.getDelta()
	
	if(self.decayTimer > self.decayRate) then
		-- Reset the stat timer.
		self.decayTimer = 0
		
		local stats = self.parent.stats.emotions
		
		local statMods = {
			chummy 		= 0,
			hunger	 	= 0,
			favor 		= 0,
			bully		= 0,
			peppy 		= 0,
			rancorous	= 0
		}
		
		--
		-- Reduce hunger.
		--
		statMods.hunger = -self.hungerSaturation
		
		if(stats.hunger < 2.5) then
			Game.sound:play(Game.assets.audio.heartbeat, "sfx")
		end
		
		if(stats.rancorous > 50) then
			statMods.hunger = statMods.hunger - math.lerp(0, 1, (stats.rancorous - 50) / 50)
		end
		
		--
		-- If our boy is dead, we should poof him.
		--
		
		--
		-- If it's been a while since click, then we should turn down the anger.
		--
		if(self.lastClick > 1) then
			statMods.bully = statMods.bully - 1
		end
		
		--
		-- If there's trash, drop the peppy.
		--
		if(#Game.state.trash > 0) then
			statMods.peppy = statMods.peppy - (#Game.state.trash * 0.1)
		end
		
		--
		-- If you have earned favor, have the favor go up.
		--
		if(stats.hunger > 25)
		and(stats.peppy > 25) then 
			statMods.favor = statMods.favor + 1
		end
		
		--
		-- If your boy is hungery, he will get Angery.
		--
		if(self.parent.stats.emotions.hunger < 30) then
			statMods.bully = statMods.bully + 0.5
		end 
		
		--
		-- Adjust surface temperature accordingly.
		--
		if(stats.bully > 50) then
			statMods.rancorous = statMods.rancorous + 1
		 	statMods.favor = statMods.favor - 0.1
		elseif(stats.bully < 50 and self.lastClick > 4) then
			statMods.rancorous = statMods.rancorous - 1
		end
		
		if(stats.rancorous > 50) then
			statMods.favor = -1
		end
		
		if(stats.rancorous < 50) and (stats.bully < 50) and (stats.favor > 25) then
			statMods.chummy = statMods.chummy + math.lerp(1, 5, stats.favor / 100)
		end
		
		for name, value in pairs(statMods) do
			self.parent.stats.emotions[name] = self.parent.stats.emotions[name] + value
		end
		
		if(stats.hunger <= 0) then
			
			for x = 1, 10 do 
				Game.particles:moveTo(self.parent.x, self.parent.y - 5 + x)
				Game.particles:emit(50)
			end
			
			self.dead = true
			Game.ecs.entities:remove(self.parent)
			return false
		end
		
		if(stats.chummy > 80) then
			if(table.length(self.evolveButtons) == 0) then
				local buttons = {}
				
				if(self.parent.stats.evolutionLevel == 0) then
					buttons = {
						{
							label = Game.assets.images.run,
							callback = function(self, dt) 
								Game.ecs.entities:get('boy'):get('BoyBehavior'):setState(Game.definitions.BoyState.BUMP)
							end
						},
						{
							label = Game.assets.images.beauty,
							callback = function(self, dt)
								Game.ecs.entities:get('boy'):get('BoyBehavior'):setState(Game.definitions.BoyState.PANG)
							end
						},
						{
							label = Game.assets.images.sunshine,
							callback = function(self, dt)
								Game.ecs.entities:get('boy'):get('BoyBehavior'):setState(Game.definitions.BoyState.TALL)
							end
						}
					}
				else
					buttons = {
						{
							label = Game.assets.images.evolve,
							callback = function(self, dt)
								Game.ecs.entities:get('boy'):get('BoyBehavior'):setState(Game.definitions.BoyEvolve[Game.ecs.entities:get('boy').stats.boyState])
							end
						}
					}
				end
				
				local buttonsDone = 0
				
				for id, button in pairs(buttons) do
					buttonsDone = buttonsDone + 1
					local buttonEntity = Game.ecs.entity()
					buttonEntity:add('MenuBehavior')
					buttonEntity:get('MenuBehavior').onLeftClick = button.callback
					
					buttonEntity.hitbox.width = button.label:getWidth()
					buttonEntity.hitbox.height = button.label:getHeight()
					buttonEntity.hitbox.offset.x = -(button.label:getWidth() / 2)
					buttonEntity.hitbox.offset.y = -(button.label:getHeight() / 2)
					
					buttonEntity.Sprite = Game.ecs.component('Sprite')
					buttonEntity.Sprite.imageData = button.label
					buttonEntity.Sprite.layer = "interface"
					
					buttonEntity.x = 670 
					buttonEntity.y = 200 + (buttonsDone * 80)
					
					table.insert(self.evolveButtons, buttonEntity)
					Game.ecs.entities:add(buttonEntity)
				end
			end
		else 
			if(table.length(self.evolveButtons) > 0) then
				for id, buttons in pairs(self.evolveButtons) do
					Game.ecs.entities:remove(buttons)
				end
				
				self.evolveButtons = {}
			end
		end
	end

	for name, value in pairs(self.parent.stats.emotions) do
		self.parent.stats.emotions[name] = math.clamp(value, 0, 100)
	end	
	
	if(self.parent.stats.emotions.rancorous > 80) then
		if(Game.state.clickValue > 0) then
			Game.state.clickValue = -Game.state.clickValue
		end
	else
		Game.state.clickValue = math.abs(Game.state.clickValue)
	end
end

function BoyBehavior:status()
	if(self.parent.stats.boyState == Game.definitions.BoyState.EGG) then
		if(Game.state.clicks >= 25) then
			Game.state.status = Game.strings.Status.EggReady
			
			if(self.hatchButton == nil) then
				self.hatchButton = Game.ecs.entity()
				self.hatchButton:add('MenuBehavior')
				self.hatchButton:get('MenuBehavior').onLeftClick = function(self, dt)
					Game.state.clicks = Game.state.clicks - 25
					Game.ecs.entities:get('boy'):get('BoyBehavior'):setState(Game.definitions.BoyState.CHUMP)
					Game.ecs.entities:get('boy'):get('BoyBehavior').hatchButton = nil
					Game.ecs.entities:get('boy'):get('AnimationBehavior').isPaused = false
					Game.ecs.entities:remove(self.parent)
					Game.state.status = Game.strings.Status.ChumpHatched
					print('Boy state change!')
				end
				self.hatchButton.Sprite = Game.ecs.component('Sprite')
				self.hatchButton.Sprite.imageData = Game.assets.images.hatch
				self.hatchButton.Sprite.layer = 'interface'
				
				self.hatchButton.hitbox.width = Game.assets.images.hatch:getWidth()
				self.hatchButton.hitbox.height = Game.assets.images.hatch:getHeight()
				self.hatchButton.hitbox.offset.x = -(Game.assets.images.hatch:getWidth() / 2)
				self.hatchButton.hitbox.offset.y = -(Game.assets.images.hatch:getHeight() / 2)
				
				self.hatchButton.x = 670
				self.hatchButton.y = 270
				
				Game.particles:moveTo(670, 270)
				Game.particles:emit(50)
				
				Game.ecs.entities:add(self.hatchButton)
			end
			
		elseif(self.parent.stats.emotions.rancorous > 80) then
			Game.state.status = Game.strings.Status.Overheat
		else
			Game.state.status = Game.strings.Status.EggStart
			
			if(self.hatchButton ~= nil) then
				Game.ecs.entities:remove(self.hatchButton)	
				self.hatchButton = nil
				Game.particles:moveTo(670, 270)
				Game.particles:emit(50)
			end
		end
	elseif(self.parent.stats.boyState == Game.definitions.BoyState.CHUMP) then
		Game.state.status = Game.strings.Status.Content
		
		if(self.parent.stats.emotions.chummy > 80) then
			Game.state.status = Game.strings.Status.ChummyChump
		end
		
		if(self.parent.stats.emotions.hunger > 80) then
			Game.state.status = Game.strings.Status.FullBoy
		end
		
		if(self.lastClick > 15 and self.parent.stats.emotions.bully < 50) then
			Game.state.status = Game.strings.Status.ClickHungry 	
		end
		
		if(self.lastClick > 30 and self.parent.stats.emotions.bully < 50) then
			Game.state.status = Game.strings.Status.ClickStarve
		end
		
		if(#Game.state.trash > 0) then
			Game.state.status = Game.strings.Status.MessyRoom
		end
		
		if(self.parent.stats.emotions.rancorous > 80) then
			Game.state.status = Game.strings.Status.Overheat	
		end
		
		if(self.parent.stats.emotions.hunger < 30 and #Game.state.food == 0) then
			Game.state.status = Game.strings.Status.HungeryBoy
		end
	end
	
	if(self.clicked == false) then
		Game.state.status = Game.strings.Status.EggUnclicked	
	end
	
	if(#self.evolveButtons >= 1) then
		if(self.parent.stats.evolutionLevel >= 1) then
			Game.state.status = Game.strings.Status.Evolve
		else
			Game.state.status = Game.strings.Status.BoyPath
		end
	end
	
	if(self.dead == true) then
		Game.state.status = Game.strings.Status.DeadBoy	
	end
end

function BoyBehavior:setState(boyState) 
	self.parent.stats.boyState = boyState
	
	Game.sound:play(Game.assets.audio.evolve, "sfx")
	
	if(Game.definitions.BoyState[boyState] ~= nil) then
		self.parent.Sprite.imageData = Game.assets.images.boys[Game.assets.text.boys[boyState].Sprite]
		self.parent.Sprite.origin.x = Game.assets.text.boys[boyState].SpriteOriginX
		self.parent.Sprite.origin.y = Game.assets.text.boys[boyState].SpriteOriginY
		
		self.parent.hitbox.width = Game.assets.text.boys[boyState].HitboxWidth
		self.parent.hitbox.height = Game.assets.text.boys[boyState].HitboxHeight
		self.parent.hitbox.offset.x = -(Game.assets.text.boys[boyState].HitboxWidth * self.parent.Sprite.origin.x)
		self.parent.hitbox.offset.y = -(Game.assets.text.boys[boyState].HitboxHeight * self.parent.Sprite.origin.y)
		
		--
		-- Reassign animations here because fuck everything.
		--
		local animations = {
			idle = {
				Game.assets.images.boys[Game.assets.text.boys[boyState].Sprite:lower()]
			},
			walk = {
				Game.assets.images.boys[Game.assets.text.boys[boyState].Sprite:lower() .. '_walk1'],
				Game.assets.images.boys[Game.assets.text.boys[boyState].Sprite:lower() .. '_walk2'],
				Game.assets.images.boys[Game.assets.text.boys[boyState].Sprite:lower() .. '_walk3'],
			}
		}
		
		self.parent.stats.evolutionLevel = self.parent.stats.evolutionLevel + 1
		
		for name, frames in pairs(animations) do	
			self.parent:get('AnimationBehavior').animations[name].frames = frames
		end
		
		for name, value in pairs(Game.assets.text.boys[boyState]) do
			if(self.parent.stats.emotions[name] ~= nil) then
				self.parent.stats.emotions[name] = value
			end
		end
		
		Game.physics:remove(self.parent.Hitbox.shape)
		self.parent.Hitbox.shape = Game.physics:rectangle(self.parent.x, self.parent.y, self.parent.hitbox.width, self.parent.hitbox.height)
		
		self.parent.velocity.y = -25
	end
	
	Game.particles:moveTo(self.parent.x, self.parent.y)
	Game.particles:emit(250)
	Game.state.status = string.format(Game.strings.Status.EvolvedBoy, self.state)
	self.evolving = true
end

function BoyBehavior:onHover(dt)
	love.mouse.setCursor(love.mouse.getSystemCursor('hand'))
end

function BoyBehavior:onLeave(dt)
	love.mouse.setCursor(love.mouse.getSystemCursor('arrow'))
end

function BoyBehavior:onLeftClick(dt)
	self.clicked = true
	
	Game.state.clicks = Game.state.clicks + Game.state.clickValue
	
	if(not self.isShaking) then
		if(math.random(1, 2) == 1) then
			self.parent.Sprite.rotation = math.rad(5)
		else
			self.parent.Sprite.rotation = -math.rad(5)
		end
	end
	
	local keys = {}
	for key, value in pairs(Game.strings.Click) do
		table.insert(keys, key)
	end
	popOff(Game.strings.Click[keys[math.random(1, #keys)]], Game.mouse.x, Game.mouse.y)
	Game.sound:play(Game.assets.audio.staple_gun, "sfx", Game.settings.volume.master * Game.settings.volume.sfx)
	
	self.parent.stats.emotions.bully = self.parent.stats.emotions.bully + 1
	self.parent.stats.emotions.rancorous = self.parent.stats.emotions.rancorous + 0.2
end

return BoyBehavior