--
--
--
--
--

local class = require 'lib.30log'

local SceneManager = class('SceneManager', {
	activeScene = 'menu',
	sceneList = {'menu', 'play', 'options', 'about'},
	sceneHistory = {'menu'},
	sceneDir = 'game.scenes'
})

function SceneManager:init()
	self.scenes = {}
	
	for _, sceneName in pairs(self.sceneList) do
		print('Loading scene ' .. sceneName)
		self.scenes[sceneName] = require(self.sceneDir .. '.' .. sceneName):new(false)
	end
end

function SceneManager:reloadScene(scene)
	self.scenes[scene] = require(self.sceneDir .. '.' .. scene):new()
end

function SceneManager:update(dt)
	self.scenes[self.activeScene]:update(dt)
end

function SceneManager:draw(dt)
	self.scenes[self.activeScene]:draw(dt)
end

function SceneManager:setScene(scene)
	local sceneExists = false
	
	if(scene == self.activeScene) then
		return false	
	end
	
	for _, sceneName in pairs(self.sceneList) do
		if scene == sceneName then
			sceneExists = true
		end
	end
	
	if(sceneExists) then
		Game.particles:reset()
		Game.mouse = {
			cursor = 'arrow',
			x = love.mouse.getX(),
			y = love.mouse.getY(),
			leftClick = false,
			rightClick = false,
			middleClick = false,
			mouseWheelUp = false,
			mouseWheelDown = false
		}
		Game.sound:stop("background")
		love.mouse.setCursor(love.mouse.getSystemCursor('arrow'))
		table.insert(self.sceneHistory, scene)
		self.activeScene = scene
		print('Moving to scene '.. scene)
	end
end

function SceneManager:lastScene()
	return self.sceneHistory[#self.sceneHistory - 1]
end

return SceneManager:new()