--
--
--

local MenuWorld = Game.ecs.world:extend('MenuWorld')

local MenuSystem = Game.ecs.system:extend('MenuSystem')

MenuSystem.quoteY = 40
MenuSystem.quoteDir = -0.5

function MenuSystem:startDraw(dt)
	love.graphics.draw(Game.assets.images.TitleScreen, (love.graphics.getWidth() / 2) - (Game.assets.images.TitleScreen:getWidth() / 2), 40)
	love.graphics.draw(Game.assets.images.Oh, 85, math.floor(self.quoteY))
	
	love.graphics.print(Game.strings.Game.Version, 4, love.graphics.getHeight() - 32)
end

function MenuSystem:update(dt)
	if(self.startedMusic == nil) then
		Game.sound:playLooping(Game.assets.audio.boyshine, "background", nil, Game.settings.volume.master)	
		self.startedMusic = true
	end
	
	self.quoteY = self.quoteY + ((self.quoteDir * 100) * dt)
	
	if(self.quoteY ~= math.clamp(self.quoteY, 40, 60)) then
		self.quoteDir = -self.quoteDir
	end
	
	self.quoteY = math.clamp(self.quoteY, 40, 60)
end

local labels = {
	{
		text = Game.strings.Menu.StartGame,
		onClick = function(self, dt)
			Game.worlds:setScene('play')
		end
	},
	{
		text = Game.strings.Menu.Options,
		onClick = function(self, dt)
			Game.worlds:setScene('options')
		end
	},
	{
		text = Game.strings.Menu.Quit,
		onClick = function(self, dt)
			love.event.quit()
		end
	}
}

local labelsDone = 0

for _, label in pairs(labels) do
	local labelEntity = Game.ecs.entity()
	labelEntity:add('MenuBehavior')
	labelEntity:get('MenuBehavior').onLeftClick = label.onClick
	labelEntity.Text = Game.ecs.component('Text')
	labelEntity.Text.text = label.text
	
	labelEntity.hitbox.width = Game.fonts.game:getWidth(label.text)
	labelEntity.hitbox.height = Game.fonts.game:getHeight()

	labelEntity.x = 300
	labelEntity.y = 400 + (labelsDone * 50)
	
	labelEntity.Text.size = 1
	
	MenuWorld:addEntity(labelEntity)
	labelsDone = labelsDone + 1
end

MenuWorld:addSystem(MenuSystem)
MenuWorld:addSystem(require 'game.systems.TextRender':new())

return MenuWorld