--
--
--

local OptionsWorld = Game.ecs.world:extend('OptionsWorld')

local OptionsSystem = Game.ecs.system:extend('OptionsSystem')

function OptionsSystem:update(dt)
	if(self.startedMusic == nil) then
		Game.sound:playLooping(Game.assets.audio.you_are_my_boy, "background", nil, Game.settings.volume.master)	
		self.startedMusic = true
	end
	
end	

function OptionsSystem:startDraw(dt)
	love.graphics.print(Game.strings.Options.OptionsLabel, (love.graphics.getWidth() / 2) - ((#Game.strings.Options.OptionsLabel * 32) / 2), 40)
	
	love.graphics.print(Game.strings.Options.ColorLabel, 260, 100)
	love.graphics.print(Game.strings.Options.R, 150, 140)
	love.graphics.print(Game.settings.colorLayer.r, 400, 140)
	
	love.graphics.print(Game.strings.Options.G, 150, 180)
	love.graphics.print(Game.settings.colorLayer.g, 400, 180)
	
	love.graphics.print(Game.strings.Options.B, 150, 220)
	love.graphics.print(Game.settings.colorLayer.b, 400, 220)
	
	love.graphics.print(Game.strings.Options.VolumeLabel, 320, 300)
	
	love.graphics.print(Game.strings.Options.SoundVolume, 150, 340)
	love.graphics.print(Game.settings.volume.sfx * 100, 400, 340)
	
	love.graphics.print(Game.strings.Options.MusicVolume, 150, 380)
	love.graphics.print(Game.settings.volume.music * 100, 400, 380)
	
	love.graphics.print(Game.strings.Options.MasterVolume, 150, 420)
	love.graphics.print(Game.settings.volume.master * 100, 400, 420)
end

local labels = {
	{
		x = 380,
		y = 140,
		text = '<',
		callback = function(self, dt)
			Game.settings.colorLayer.r = Game.settings.colorLayer.r - 5
		end
	},	
	{
		x = 380,
		y = 180,
		text = '<',
		callback = function(self, dt)
			Game.settings.colorLayer.g = Game.settings.colorLayer.g - 5
		end
	},
	{
		x = 380,
		y = 220,
		text = '<',
		callback = function(self, dt)
			Game.settings.colorLayer.b = Game.settings.colorLayer.b - 5
		end
	},
	{
		x = 450,
		y = 140,
		text = '>',
		callback = function(self, dt)
			Game.settings.colorLayer.r = Game.settings.colorLayer.r + 5
		end
	},
	{
		x = 450,
		y = 180,
		text = '>',
		callback = function(self, dt)
			Game.settings.colorLayer.g = Game.settings.colorLayer.g + 5
		end
	},
	{
		x = 450,
		y = 220,
		text = '>',
		callback = function(self, dt)
			Game.settings.colorLayer.b = Game.settings.colorLayer.b + 5
		end
	},
	{
		x = 380,
		y = 340,
		text = '<',
		callback = function(self, dt)
			Game.settings.volume.sfx = Game.settings.volume.sfx - 0.01
		end
	},	
	{
		x = 380,
		y = 380,
		text = '<',
		callback = function(self, dt)
			Game.settings.volume.music = Game.settings.volume.music - 0.01
		end
	},
	{
		x = 380,
		y = 420,
		text = '<',
		callback = function(self, dt)
			Game.settings.volume.master = Game.settings.volume.master - 0.01
		end
	},
	{
		x = 450,
		y = 340,
		text = '>',
		callback = function(self, dt)
			Game.settings.volume.sfx = Game.settings.volume.sfx + 0.01
		end
	},
	{
		x = 450,
		y = 380,
		text = '>',
		callback = function(self, dt)
			Game.settings.volume.music = Game.settings.volume.music + 0.01
		end
	},
	{
		x = 450,
		y = 420,
		text = '>',
		callback = function(self, dt)
			Game.settings.volume.master = Game.settings.volume.master + 0.01
		end
	},
	{
		x = 100,
		y = 500,
		text = 'Save',
		callback = function(self, dt)
			popOff(Game.strings.Options.SavedSettings, Game.mouse.x, Game.mouse.y)
			Game.file.save('settings.ini', Game.settings)
		end
	},
	{
		x = 400,
		y = 500,
		text = 'Return to menu',
		callback = function(self, dt)
			Game.worlds:setScene('menu')
		end
	}
}

for _, label in pairs(labels) do
	local labelEntity = Game.ecs.entity()
	labelEntity.x = label.x
	labelEntity.y = label.y
	
	labelEntity.hitbox.width = Game.fonts.game:getWidth(label.text)
	labelEntity.hitbox.height = Game.fonts.game:getHeight()
	
	labelEntity:add('MenuBehavior')
	labelEntity:get('MenuBehavior').onLeftClick = label.callback
		
	labelEntity.Text = Game.ecs.component('Text')
	labelEntity.Text.text = label.text
	
	OptionsWorld:addEntity(labelEntity)
end

OptionsWorld:addSystem(OptionsSystem)
OptionsWorld:addSystem(require 'game.systems.TextRender':new())

return OptionsWorld