--
--
--

local PlayWorld = Game.ecs.world:extend('PlayWorld')

local PlaySystem = Game.ecs.system:extend('PlaySystem')

PlaySystem.buttons = {}

function PlaySystem:update(dt)
	local boy = Game.ecs.entities:get('boy')
	
	if(boy ~= nil) then
		Game.state.clickValue = 6 + math.floor(math.lerp(-5, 5, (boy.stats.emotions.favor / 100)))
		
		if(#boy:get('BoyBehavior').evolveButtons > 0) then
			for _, button in pairs (self.buttons) do 
				button.x = 1000
			end
		else 
			for _, button in pairs(self.buttons) do
				button.x = 670
			end
		end
		
		if(boy.stats.boyState ~= Game.definitions.BoyState.EGG) then
			if(#self.buttons == 0) then
				for name, label in pairs(Game.assets.text.items) do
					print(string.format('Registering item %s', name))
					
					local button = Game.ecs.entity:new()
					button:add('MenuBehavior')
					button:get('MenuBehavior').onLeftClick = function(self, dt)
						if(Game.state.clicks >= self.parent.cost and self.parent.activeCooldown >= self.parent.cooldown) then
							popOff('-' .. self.parent.cost, 50, 50)
							Game.particles:emit(math.random(1, 5) * 10)
							local item = (require(string.format('game.entities.%s', label.item))):new()
							item.x = 50
							item.y = 50
							
							item.velocity.x = math.random(5, 10)
							item.drag.x = 0.2
					
							Game.worlds.scenes.play:addEntity(item)
							Game.state.clicks = Game.state.clicks - self.parent.cost
							self.parent.activeCooldown = 0
						end
					end
					
					button.x = 670
					button.y = 270 + ((label.id - 1) * 50)
					
					button.Sprite = Game.ecs.component('Sprite')
					button.Sprite.imageData = Game.assets.images.ui[label.button]
					button.Sprite.layer = "interface"
					
					button.cost = label.cost
					button.spriteRef = Game.assets.images.ui[label.button]
					
					button.cooldown = label.cooldown
					button.activeCooldown = label.cooldown
					
					button.update = function(self, dt)
						self.class.update(self, dt)
						
						self.activeCooldown = self.activeCooldown + dt
						
						if(Game.state.clicks < self.cost or self.activeCooldown < self.cooldown) then
							self.Sprite.imageData = Game.assets.images.ui.locked
						else
							self.Sprite.imageData = self.spriteRef
						end
					end
				
					local sprite = Game.assets.images.ui[label.button]
				
					button.hitbox.width = sprite:getWidth()
					button.hitbox.height = sprite:getHeight()
					
					button.hitbox.offset.x = -math.floor(sprite:getWidth() / 2)
					button.hitbox.offset.y = -math.floor(sprite:getHeight() / 2)
					
					table.insert(self.buttons, label.id, button)
					Game.ecs.entities:add(button)
				end
			end
		else
			if(#self.buttons > 0) then
				for _, button in pairs(self.buttons) do
					Game.ecs.entities:remove(button)
				end
				
				self.buttons = {}
			end
		end
	end
end

function PlaySystem:startDraw(dt)
	love.graphics.setCanvas(Game.layers.background)
	love.graphics.draw(Game.assets.images.backgrounds[Game.state.setting .. '_bg'], 25, 20)
	love.graphics.setCanvas(Game.layers.foreground)
	love.graphics.draw(Game.assets.images.backgrounds[Game.state.setting .. '_fg'], 25, 20)
	
	love.graphics.setCanvas(Game.layers.interface)
	love.graphics.draw(Game.assets.images.play_interface)
	
	local boy = Game.ecs.entities:get('boy')
	
	if(boy ~= nil) then
		local emotionsDone = 0
		
		for name, value in pairs(boy.stats.emotionList) do
			if(boy.stats.boyState == Game.definitions.BoyState.EGG) then
				love.graphics.print("???", 720, 30 + (emotionsDone * 28))
			else
				love.graphics.rectangle("fill", 581, 45 + (emotionsDone * 28), math.round(180 * (boy.stats.emotions[value] / 100)), 10)
			end
			
			emotionsDone = emotionsDone + 1
		end
	end
	
	love.graphics.print(Game.state.clicks, 400 - Game.fonts.game:getWidth(tostring(Game.state.clicks)), 550)
	love.graphics.printf(Game.state.status, 280 - (Game.fonts.game:getWidth(Game.state.status) / 2), 475, 600)
	love.graphics.print(Game.strings.Play.ClickLabel, 416, 550)
	love.graphics.setCanvas()
end

local Boy = require 'game.entities.boy':new()

local Mouse = Game.ecs.entity()
Mouse:add('MouseBehavior')

local ExitButton = Game.ecs.entity:extend('ExitButton', {
	behaviorList = {'MenuBehavior'}
}):new()
ExitButton:get('MenuBehavior').onLeftClick = function(self, dt)
	Game.worlds:setScene('menu')
end
ExitButton.Text = Game.ecs.component('Text')
ExitButton.Text.layer = "interface"
ExitButton.Text.text = 'Exit'
ExitButton.x = 5
ExitButton.y = 600 - Game.fonts.game:getHeight()
ExitButton.hitbox = {
	offset = {
		x = 0, y = 0
	},
	width = Game.fonts.game:getWidth('Exit'),
	height = Game.fonts.game:getHeight()
}

PlaySystem.walls = {
	left = {},
	right = {},
	bottom = {}
}

PlaySystem.walls.left = Game.physics:rectangle(0, 0, 20, 600)
PlaySystem.walls.left.still = true
PlaySystem.walls.right = Game.physics:rectangle(550, 0, 250, 600)
PlaySystem.walls.right.still = true
PlaySystem.walls.bottom = Game.physics:rectangle(0, Game.state.floor_y, 600, 200)
PlaySystem.walls.bottom.still = true

Game.physics:register(PlaySystem.walls.left)
Game.physics:register(PlaySystem.walls.right)
Game.physics:register(PlaySystem.walls.bottom)

PlayWorld:addEntity(Mouse, 'mouse')
PlayWorld:addEntity(ExitButton, 'exit_button')
PlayWorld:addEntity(Boy, 'boy')
PlayWorld:addSystem(PlaySystem:new())
PlayWorld:addSystem(require 'game.systems.ItemTracking':new())
PlayWorld:addSystem(require 'game.systems.Physics':new())
PlayWorld:addSystem(require 'game.systems.SpriteRender':new())
PlayWorld:addSystem(require 'game.systems.TextRender':new())

return PlayWorld