--
--
--

local Physics = Game.ecs.system:extend('Physics', {})

function Physics:update(dt)
	--Game.physics:update(dt)
end

function Physics:filter(entity)
	return (entity.Hitbox ~= nil)
end

function Physics:process(entity, dt)
	if(entity.Hitbox.initialized == false) then
		entity.Hitbox.shape = Game.physics:rectangle(entity.x, entity.y, entity.hitbox.width, entity.hitbox.height)
		entity.Hitbox.shape.parent = entity
		Game.physics:register(entity.Hitbox.shape)
		entity.Hitbox.initialized = true
	end
	
	if(not entity.Hitbox.still) then
		if(entity.velocity.min.x ~= 0)
		or(entity.velocity.min.y ~= 0)
		or(entity.velocity.max.x ~= 0)
		or(entity.velocity.max.y ~= 0)
		then
			entity.velocity.x = math.clamp(entity.velocity.x, entity.velocity.min.x, entity.velocity.max.x)
			entity.velocity.y = math.clamp(entity.velocity.y, entity.velocity.min.y, entity.velocity.max.y)
		end
		
		if(math.abs(entity.velocity.x) > 0) then
			entity.velocity.x = math.rsign(entity.velocity.x) * (math.abs(entity.velocity.x) - entity.drag.x)
			
			if(math.abs(entity.velocity.x) < math.abs(entity.drag.x)) then
				entity.velocity.x = 0
			end
		end
		
		if(math.abs(entity.velocity.y) < (Game.state.gravity * dt)) then
			entity.velocity.y = math.rsign(entity.velocity.y) * (entity.velocity.y + (Game.state.gravity * dt))
		end
		
		entity.Hitbox.shape:move(entity.velocity.x, entity.velocity.y)
	end
	
	local collisions = Game.physics:collisions(entity.Hitbox.shape)
	
	for other, separating_vector in pairs(collisions) do
		local otherStatic = false
		local selfStatic = false
		
		if(entity.still) then
			selfStatic = true
		end
		
		if(entity.Hitbox.still) then
			selfStatic = true
		end	

	
		if(other.still) then
			otherStatic = true
		end
		
		if(other.parent) then
			if(other.parent.Hitbox) then
				if(other.parent.Hitbox.still) then
					otherStatic = true
				end	
			end
		end
		
		if(otherStatic and not selfStatic) then
			entity.Hitbox.shape:move(separating_vector.x, separating_vector.y)
			entity.velocity.x = 0
			entity.velocity.y = 0
		elseif(not otherStatic and not selfStatic) then
			entity.Hitbox.shape:move(separating_vector.x / 2, separating_vector.y / 2)
			entity.velocity.x = 0
			entity.velocity.y = 0
			
			other:move(-separating_vector.x / 2, -separating_vector.y / 2)
			if(other.parent) then
				other.parent.velocity.x = 0
				other.parent.velocity.y = 0 
			end
		elseif(not otherStatic and selfStatic) then
			if(other.parent) then
				other.parent.velocity.x = 0
				other.parent.velocity.y = 0 
			end
			other:move(-separating_vector.x,  -separating_vector.y)
		end
	end
	
	entity.x, entity.y = entity.Hitbox.shape:center()
	entity.y = entity.y + ((1 - entity.hitbox.offset.y) / 2)
end

function Physics:startDraw(dt)
	if(Game.debug.showBoxes) then
		love.graphics.setCanvas(Game.layers.debug)
		
		for _, shape in pairs(Game.physics.hash:shapes()) do
			shape:draw('line')
		end
		
		love.graphics.setCanvas()
	end
end

function Physics:draw(entity, dt)

end

return Physics